#!/bin/bash

### [ ENV ] ###
SCRIPT_DIR="$(dirname "$(readlink -f "${0}")")"
PARENT_DIR="$(dirname "${SCRIPT_DIR}")"
GET_IPXE="${SCRIPT_DIR}/get_ipxe.sh"

### [ DEFAULTS ] ###
NBM_USER="niobium"
INSTALL_DIR="/srv/nbm"
DHCPD_BIN="/usr/sbin/dhcpd"
DHCPD_CONF="/etc/dhcpd.conf"
if [ ! -f "${DHCPD_CONF}"]; then
    DHCPD_CONF="/etc/dhcp/dhcpd.conf"
fi
DHCPD_DEF="/etc/default/isc-dhcp-server"
IPXE_KERNEL="undionly.kpxe"
TFTP_DIR="${INSTALL_DIR}/tftp"
TFTP_CONF="/etc/default/tftpd-hpa"

### [ FUNCTIONS ] ###
. ${SCRIPT_DIR}/functions.sh

isc_dhcp_server_installed ()
{
    command_exists ${DHCPD_BIN} &&
        ${DHCPD_BIN} --version 2>&1 | grep -q isc-dhcp
}

### [ SCRIPT ] ###
# Run script as root
if [ $(id -u) -ne 0 ]; then
    log "${0}: Must be run as root"
    exit 1
fi

# Create user if not exists
id -u "${NBM_USER}" &>/dev/null || useradd "${NBM_USER}"

# Create symbolic link if not installed in INSTALL_DIR
if [ ! -d "${INSTALL_DIR}" ]; then
    log "Creating symbolic link from \"${PARENT_DIR}\" to \"${INSTALL_DIR}\""
    ln -s "${PARENT_DIR}" "${INSTALL_DIR}"
else
    log "The directory \"${INSTALL_DIR}\" already exists"
fi

# Grant permissions (FIX!!!)
echo "cristian ALL=(root) NOPASSWD: /usr/bin/bash -" > "/etc/sudoers.d/${NBM_USER}"
chmod 0440 "/etc/sudoers.d/${NBM_USER}"

# Make "builds" dir if not exists
mkdir -p ${INSTALL_DIR}/builds
chown ${NBM_USER}:${NBM_USER} "${INSTALL_DIR}/builds"

##
# DHCP Server
##
log "Checking if ISC DHCP Server is installed..."
if ! isc_dhcp_server_installed; then
    warn "ISC DHCP server is required by Niobium:"
    warn "  # pacman -S dhcp"
    warn "  # apt-get install isc-dhcp-server"
    die 'Aborting installation...'
else
    log "Installed!"
fi

# Condigure DHCP Server 
log "Configuring DHCP Server..."
NBM_DHCPD_CONF="${INSTALL_DIR}/dhcpd.conf"
NBM_DHCP_INCLUDE="include \"${NBM_DHCPD_CONF}\";"
# Append include statement in DHCPD_CONF

if ! grep -q -F "${NBM_DHCP_INCLUDE}" "${DHCPD_CONF}"; then
    log "Adding \"${NBM_DHCPD_CONF}\" to \"${DHCPD_CONF}\""
    echo "${NBM_DHCP_INCLUDE}" >> "${DHCPD_CONF}"
else
    log "The file \"${DHCPD_CONF}\" already contains \"${NBM_DHCPD_CONF}\""
fi

if [ -f "${DHCPD_DEF}" ]; then
    if ask "Found \"${DHCPD_DEF}\" file, add \"br0\" interface? (for testing purposes)" N; then
        BRIDGE="br0"
        log "Detected \"${DHCPD_DEF}\" file, adding interface \"${BRIDGE}\" by default."
        sed -i -e "s|^INTERFACES=\"\(.*\)\"$|INTERFACES=\"\1 ${BRIDGE}\"|" ${DHCPD_DEF}
    fi
fi

# Touch config file
log "Creating default config file: \"${NBM_DHCPD_CONF}\""
touch "${NBM_DHCPD_CONF}"
chown ${NBM_USER}:${NBM_USER} "${NBM_DHCPD_CONF}"
# cat > "${NBM_DHCPD_CONF}" <<-"EOF"
# 	### Automatically-generated file. Do not edit! ###
# 	# Subnet declaration
# 	subnet 10.42.0.0 netmask 255.255.255.0 {
# 	    # Subnet config
# 	    range 10.42.0.10 10.42.0.30;
# 	    option routers 10.42.0.1;
# 	    option broadcast-address 10.42.0.255;
# 	    option subnet-mask 255.255.255.0;
# 	    option domain-name-servers 10.42.0.1, 8.8.8.8, 8.8.4.4;
# 	    
# 	    # Hosts config
# 	    host pc-01 {
# 	        option host-name "pc-01";
# 	        hardware ethernet 52:54:00:12:34:61;
# 	        fixed-address 10.42.0.31;
# 	    }
# 	    host pc-02 {
# 	        option host-name "pc-02";
# 	        hardware ethernet 52:54:00:12:34:62;
# 	        fixed-address 10.42.0.32;
# 	    }
# 	    
# 	    # iPXE config
# 	    if exists user-class and option user-class = "iPXE" {
# 	        #filename "http://boot.ipxe.org/demo/boot.php";
# 	        filename "http://10.42.0.1:1337/boot";
# 	    } else {
# 	        filename "undionly.kpxe";
# 	    }
# 	    next-server 10.42.0.1;
# 	}
# EOF

log "Restarting DHCP Server"
# Restart DHCP Server
systemctl restart dhcpd4 \
    || service isc-dhcp-server restart
    
##
# TFTP Server
##
log "Checking if HPA's TFTP is installed..."
if ! command_exists in.tftpd; then
    warn "HPA's TFTP is required by Niobium:"
    warn "  # pacman -S tftp-hda"
    warn "  # apt-get install tftpd-hpa"
    die 'Aborting installation...'
else
    log "Installed!"
fi

# Configure TFTP Server
log "Configuring TFTP Server..."
if [ ! -f ${TFTP_DIR} ]; then
    log "Creating directory \"${TFTP_DIR}\""
    mkdir -p "${TFTP_DIR}"
else
    log "The directory \"${TFTP_DIR}\" already exists"
fi

if [ -f "${TFTP_CONF}" ]; then
    log "TFTP config file detected, add manually these lines to \"${TFTP_CONF}\""
    hr
    cat <<-"EOF"
		TFTP_USERNAME="tftp"
		TFTP_DIRECTORY="/srv/tftp"
		TFTP_ADDRESS="0.0.0.0:69"
		TFTP_OPTIONS="--secure"
EOF
    hr
fi

log "Maybe you want to edit manually /etc/systemd/system/tftpd.service or /usr/lib/systemd/system/tftpd.service"
hr
cat <<-EOF
	[Unit]
	Description=hpa's original TFTP daemon
	
	[Service]
	ExecStart=/usr/sbin/in.tftpd -s ${TFTP_DIR}
	StandardInput=socket
	StandardOutput=inherit
	StandardError=journal
EOF
hr

log "Getting iPXE kernel..."
if [ ! -f "${TFTP_DIR}/${IPXE_KERNEL}" ]; then
    log "iPXE kernel \"${IPXE_KERNEL}\" not found in \"${TFTP_DIR}\", obtaining..."
    if ! ${GET_IPXE} "${IPXE_KERNEL}" "${TFTP_DIR}"; then
        die "No se ha podido generar el kernel de ipxe."
    fi
else
    log "The iPXE kernel \"${TFTP_DIR}/${IPXE_KERNEL}\" already exists"
fi

log "Restarting TFTP Daemon"
# Restart TFTP Server
systemctl restart tftpd.socket \
    || service tftpd-hpa restart

##
# NFS Server
##
log "Checking if NFS Server is installed..."
if ! command_exists exportfs; then
    warn "NFS Server is required by Niobium:"
    warn "  # pacman -S nfs-utils"
    warn "  # apt-get install nfs-kernel-server portmap"
    die 'Aborting installation...'
else
    log "Installed!"
fi

# Make directory /etc/exports.d if not exists
mkdir -p /etc/exports.d

# Create symlink to exports config
NBM_NFS_EXPORTS="${INSTALL_DIR}/exports"
touch "${NBM_NFS_EXPORTS}"
chown ${NBM_USER}:${NBM_USER} "${NBM_NFS_EXPORTS}"
ln -s "${NBM_NFS_EXPORTS}" /etc/exports.d/niobium.exports

##
# Node.js
##
log "Checking if Node.js is installed..."
# Check that Node.js is installed
if ! command_exists npm; then
    warn "Node.js is required by Niobium:"
    warn "  # pacman -S nodejs"
    warn "  # apt-get install nodejs npm"
    die 'Aborting installation...'
else
    log "Installed!"
fi

mkdir -p /home/${NBM_USER}/.npm
chown -R ${NBM_USER}:${NBM_USER} /home/${NBM_USER}

# Download all required Node.js modules
pushd "${INSTALL_DIR}/web"
su ${NBM_USER} -c "npm install"
popd

log "Done!"
