/**
* Client.js
*
* @description :: TODO: You might write a short summary of how this model works and what it represents here.
* @docs        :: http://sailsjs.org/#!documentation/models
*/

var Promise = require('bluebird');

module.exports = {
    
    attributes: {
        
        /**
         * Nombre a mostrar. Debe cumplir con la norma RFC 1035.
         */
        name: {
            type: 'string',
            regex: /^[a-z0-9-]{1,253}$/,
            unique: true,
            required: true,
            notNull: true,
        },
        
        /**
         * Dirección MAC del dispositivo, de la forma:
         *     00:00:00:00:00:00
         */
        mac: {
            type: 'string',
            regex: /^([0-9A-Fa-f]{2}:){5}([0-9A-Fa-f]{2})$/,
            unique: true,
            required: true,
            notNull: true
        },
        
        /**
         * Dirección IP de la máquina en IPv4 o IPv6.
         */
        ip: {
            type: 'string',
            // ip: true
            regex: /^(([1-9]?\d|1\d\d|25[0-5]|2[0-4]\d)\.){3}([1-9]?\d|1\d\d|25[0-5]|2[0-4]\d)$/,
            defaultsTo: '',
            notNull: true,
        },
        
        /**
         * IP must be assigned by DHCP Server?
         */
        
        dhcp: {
            type: 'boolean',
            defaultsTo: false
        },
        
        /**
         * Grupos a los que pertenece.
         */
        groups: {
            collection: 'clientGroup',
            via: 'clients'
        }
    },
    
    // [Funciones extra]
    
    /**
     * Get a Client and populate groups.images.connector
     *
     * @requires module:bluebird
     * @see: https://github.com/balderdashy/waterline/issues/256#issuecomment-42742446
     */
    populateAllById: function (clientId) {
        var populateImages = function (group) {
            return ClientGroup.findOne(group.id).populate('images');
        };
        
        var mapGroups = function (client) {
            if ( client && client.groups ) {
                return [client, Promise.map(client.groups, populateImages)];
            }
            return [client];
        };
        
        return Client.findOne(clientId).populate('groups')
            .then(mapGroups).spread(function setGroups(client, groups) {
                if ( client && groups ) {
                    client.groups = groups;
                }
                return client;
            });
    },
    
    /**
     * Get all distinct BinaryImages from a found Client
     */
    getBinaryImages: function (client) {
        var imgs = {};
        
        if ( client ) {
            _.forEach(client.groups, function iterateGroups(group) {
                _.forEach(group.images, function iterateImages(img) {
                    // Get if distinct and not hidden
                    if ( ! imgs[img.id] && ! img.hidden ) {
                        imgs[img.id] = img;
                    }
                });
            });
        }
        
        // Get all images as an array
        imgs = _.values(imgs);
        
        // Sort images by priority (desc)
        // sails.log.debug('pre-sort:', imgs);
        imgs = _.sortBy(imgs, function(img){
            return (-img.priority || 0);
        });
        // sails.log.debug('post-sort:', imgs);
        
        return imgs;
    },
    
    /**
     * Form builder
     */
    formFields: [{
        name: 'id',
        label: 'Id',
        type: 'text',
    }, {
        name: 'name',
        label: 'Name*',
        type: 'text',
        attrs: {
            'data-editable': true,
            'data-bv-rfc1035': 'true',
        }
    }, {
        name: 'mac',
        label: 'MAC*',
        type: 'text',
        attrs: {
            'data-writable': true,
            'data-bv-mac': 'true',
            'data-bv-notempty': 'true',
        }
    }, {
        name: 'ip',
        label: 'IP',
        type: 'text',
        attrs: {
            'data-editable': true,
            'data-bv-ip': 'true',
            'data-bv-ip-ipv4': 'true',
            'data-bv-ip-ipv6': 'false',
        }
    }, {
        name: 'dhcp',
        label: 'DHCP host',
        type: 'checkbox',
        attrs: {
            'data-editable': true,
        },
    }, {
        name: 'groups',
        label: 'Groups',
        relation: 'clientgroup',
        type: 'select',
        multiple: true,
        attrs: {
            'data-editable': true,
            'title': 'No groups assigned',
        },
        perms: 'manageGroups',
        // Must be injected: { id: '...', name: '...' }
        values: []
    }],
    
    /**
     * List Builder. Max col-md-10
     */
    listFields: [{
        name: 'id',
        label: '#',
        'class': 'col-md-1'
    }, {
        name: 'name',
        label: 'Name',
        'class': 'col-md-4'
    }, {
        name: 'mac',
        label: 'MAC',
        'class': 'col-md-3'
    }, {
        name: 'ip',
        label: 'IP',
        'class': 'col-md-2'
    }]
    
};

