/**
* User.js
*
* @description :: TODO: You might write a short summary of how this model works and what it represents here.
* @docs        :: http://sailsjs.org/#!documentation/models
*/

var bcrypt = require('bcrypt'),
    Promise = require('bluebird');

module.exports = {
    
    attributes: {
        
        /**
         * User account name
         */
        name: {
            type: 'string',
            unique: true,
            required: true,
            notNull: true
        },
        
        /**
         * User full name
         */
        fullName: {
            type: 'string',
            defaultsTo: '',
            notNull: true
        },
        
        /**
         * User email
         */
        email: {
            type: 'string',
            email: true,
            defaultsTo: '',
            notNull: true
        },
        
        /**
         * Bcrypt encrypted password
         */
        encryptedPassword: {
            type: 'string',
            defaultsTo: '',
            notNull: true
        },
        
        /**
         * User roles
         */
        roles: {
            collection: 'role',
            via: 'users',
            dominant: true
        },
        
        /**
         * Check if user satisfies given permissions.
         *
         * @param {string|array} perms - Permissions to check
         * @param {boolean} any - Check that, at least, ne permission is satisfied.
         * @return {boolean} All permissions satisfied.
         */
        hasPerm: function (perms, any) {
            var self = this;
            
            // Initialize the permissions cache (refreshed at each page reload)
            if ( typeof self._perms === 'undefined' ) {
                self._perms = [];
                
                // Iterate through al roles and build _perms list
                _.each(self.roles, function iterRoles(role) {
                    _.each(role.permissions, function iterPerms(perm) {
                        self._perms.push(perm.name);
                    });
                });
                
                // Ensure unique elements in _perms
                self._perms = _.uniq(self._perms);
            }
            
            // Ensure that 'perms' is a list...
            if ( typeof perms !== 'object' || ! perms.length ) {
                perms = perms.split(' ');
            }
            
            // And compact all values (remove all falsy values)
            perms = _.compact(perms);
            
            // Get intersection between user and queries perms
            var intxn = _.intersection(perms, self._perms);
            
            if ( !! any ) {
                // Satisfied if, at least, 1 element
                return ( intxn.length > 0 );
            } else {
                // If equal lengths, all satisfied
                return ( perms.length > 0 && perms.length === _.intersection(perms, self._perms).length );
            }
        },
        
        /**
         * Check if password is correct (synchronously).
         *
         * @param {string} password Password to compare.
         * @return {boolean} True if correct.
         */
        checkPassword: function (password) {
            return bcrypt.compareSync(password, this.encryptedPassword || '');
        },
        
        /**
         * Sobreescribimos la función toJSON para eliminar datos sensibles que 
         * no queremos que sean devueltos.
         */
        toJSON: function () {
            var obj = this.toObject();
            delete obj.encryptedPassword;
            // Evitamos que desaparezcan los roles al exportar a JSON.
            // FIX: https://github.com/balderdashy/waterline/issues/256#issuecomment-42742446
            if ( (this.roles || []).length ) {
                obj.roles = this.roles.map(function (r) {
                    return r.toJSON();
                });
            }
            return obj;
        }
        
    },
    
    beforeValidate: function (values, next) {
        // Delete _perms cache (required?)
        delete values._perms;
        
        // Fix user 'name'
        // if ( values.name ) {
        //     values.name = values.name.toLowerCase();
        // }
        
        // Fix roles array
        if ( values.hasOwnProperty('roles') ) {
            if ( values.roles ) {
                // Ensure that 'roles' is a list
                values.roles = ( typeof values.roles !== 'object' || ! values.roles.length ) ?
                    [values.roles] : values.roles;
            } else {
                values.roles = [];
            }
            
            // This should not be done this way
            if ( ! values.roles.length ) {
                delete values.roles;
            }
        }
        
        // Si password está definido...
        if ( values.password ) {
            // Calcula el bcrypt y lo guarda
            bcrypt.hash(values.password, 10, function passwordEncrypted(err, password) {
                if ( err ) {
                    return next(err);
                }
                
                // Al usar un nombre diferente conseguimos que no se ejecute en
                // cada update, save, etc
                values.encryptedPassword = password;
                next();
            });
        } else {
            next();
        }
    },
    
    // [Funciones extra]
    
    /**
     * Dado el identificador de un usuario, obtiene todos sus roles y permisos.
     *
     * @requires module:bluebird
     * @see: https://github.com/balderdashy/waterline/issues/256#issuecomment-42742446
     */
    getWithPerms: function (userId) {
        var getPerms = function (role) {
            return Role.findOne(role.id).populate('permissions');
        };
        
        var mapRoles = function (user){
            if ( user && user.roles ) {
                return [user, Promise.map(user.roles, getPerms)];
            }
        };
        
        return User.findOne(userId).populate('roles')
            .then(mapRoles)
            .spread(function(user, roles){
                user.roles = roles;
                return user;
        });
    },
    
    /**
     * Form builder
     */
    formFields: [{
        name: 'id',
        label: 'Id',
        type: 'text',
    }, {
        name: 'name',
        label: 'Name*',
        type: 'text',
        attrs: {
            'data-writable': true,
            'data-bv-stringlength': 'true',
            'data-bv-stringlength-min': 3,
            'data-bv-notempty': 'true',
        }
    }, {
        name: 'fullName',
        label: 'Full Name',
        type: 'text',
        attrs: {
            'data-editable': true,
        }
    }, {
        name: 'email',
        label: 'Email',
        type: 'text',
        attrs: {
            'data-editable': true,
            'data-bv-emailaddress': 'true',
        }
    }, {
        name: 'password',
        label: 'Password',
        type: 'password',
        attrs: {
            'data-editable': true,
            'placeholder': 'Leave blank to keep password unchanged',
            'data-bv-securepassword': 'true',
            'data-bv-identical': 'true',
            'data-bv-identical-field': 'passwordConfirmation',
        }
    }, {
        name: 'passwordConfirmation',
        label: 'Password Confirmation',
        type: 'password',
        attrs: {
            'data-editable': true,
            'data-bv-identical': 'true',
            'data-bv-identical-field': 'password',
        }
    }, {
        name: 'roles',
        label: 'Roles',
        relation: 'role',
        type: 'select',
        multiple: true,
        attrs: {
            'data-editable': true,
            'title': 'No roles assigned',
        },
        perms: 'manageRoles',
        // Must be injected: { id: '...', name: '...' }
        values: []
    }],
    
    /**
     * List Builder. Max col-md-10
     */
    listFields: [{
        name: 'id',
        label: '#',
        'class': 'col-md-1'
    }, {
        name: 'name',
        label: 'Name',
        'class': 'col-md-2'
    }, {
        name: 'fullName',
        label: 'Full Name',
        'class': 'col-md-3'
    }, {
        name: 'email',
        label: 'Email',
        'class': 'col-md-4'
    }]
};
