/**
 * GenScript
 *
 * @description :: Script generators
 */

var ejs = require('ejs'),
    path = require('path'),
    sprintf = require('sprintf').sprintf,
    dateFormat = require('dateformat'),
    self = {};

module.exports = _.merge(self, {
    
    /**
     * Render script and return result in callback.
     *
     * @param {String} path Script template.
     * @param {Object} options Data required by template.
     * @param {Function} [next] Optional callback.
     */
    _renderScript: function (fileRelPath, options, next) {
        next = next || function () {};
        var filePath = path.join(__dirname, fileRelPath + '.ejs');
        
        return ejs.renderFile(filePath, options, function fileRendered(err, result) {
            if ( err ) {
                return next(err);
            }
            
            return next(null, result);
        });
    },
    
    /**
     * boot.ipxe
     */
    boot: function (next) {
        next = next || function () {};
        
        return self._renderScript('scripts/ipxe/boot.ipxe', {
            server_ip: Config.getParam('server_ip', '${dhcp-server}'),
            server_port: Config.getParam('server_port', 1337),
        }, function scriptRendered(err, result){
            if ( err ) {
                return next(err);
            }
            
            next(null, result);
        });
    },
    
    /**
     * script.ipxe
     */
    ipxe: function (clientId, next) {
        next = next || function () {};
        
        Client.populateAllById(clientId).then(function clientFound(client) {
            // Get all distinct binary images
            imgs = Client.getBinaryImages(client);
            
            // Fix defauls
            imgs = _.filter(imgs, function iterateImages(img) {
                if ( ! img.connector ) {
                    return false;
                }
                
                var connector = Connector.get(img.connector);
                var build = img.build;
                
                _.merge(img, connector, img, {
                    url: img.url(),
                    server: Config.getParam('server_ip', '${dhcp-server}'),
                });
                
                if ( connector && typeof connector.nfsroot !== 'undefined' &&
                        typeof connector.nfsopts !== 'undefined' ) {
                    img.nfsroot = connector.path + '/' + build;
                    
                    if ( connector.nfsroot ) {
                        img.nfsroot += '/' + connector.nfsroot;
                    }
                }
                
                img.ipxe = _.map(img.ipxe, function (line) { return sprintf(line, img); });
                
                return true;
            });
            
            // sails.log.debug('imgs', imgs);
            
            return self._renderScript('scripts/ipxe/script.ipxe', {
                imgs: imgs,
                ipxe_menu_timeout: Config.getParam('ipxe_menu_timeout', 0)
            }, function scriptRendered(err, result){
                if ( err ) {
                    return next(err);
                }
                
                next(null, result);
            });
            
        }).fail(function clientFoundError(err) {
            return next(err);
        });
    },
    
    /**
     * /etc/dhcpd.conf
     */
    dhcpd: function (next) {
        next = next || function () {};
        
        Client.find({
            ip: { '!': '' },
            dhcp: true
        }).exec(function clientsFound(err, clients) {
            // Log any error and exit
            if ( err ) {
                return next(err);
            }
            
            return self._renderScript('scripts/dhcpd.conf', {
                server_ip: Config.getParam('server_ip'),
                server_port: Config.getParam('server_port', 1337),
                dhcp_subnet: Config.getParam('dhcp_subnet'),
                dhcp_mask: Config.getParam('dhcp_mask'),
                dhcp_range: Config.getParam('dhcp_range'),
                dhcp_broadcast: Config.getParam('dhcp_broadcast'),
                dhcp_dns: Config.getParam('dhcp_dns'),
                ipxe_tftp_kernel: Config.getParam('ipxe_tftp_kernel'),
                clients: clients,
            }, function scriptRendered(err, result){
                if ( err ) {
                    return next(err);
                }
                
                return next(null, result);
            });
        });
    },
    
    /**
     * /etc/exports
     */
    exports: function (next) {
        next = next || function () {};
        
        // Get all images, populate connectors and filter those with `nfsOptions`
        BinaryImage.find()
        .exec(function imagesFound(err, imgs) {
            // Log any error and exit
            if ( err ) {
                return next(err);
            }
            
            // Filter images without correct connector
            imgs = _.filter(imgs, function(img) {
                var connector = Connector.get(img.connector);
                
                if ( connector && typeof connector.nfsroot !== 'undefined' &&
                        typeof connector.nfsopts !== 'undefined' ) {
                    img.nfsroot = connector.path + '/' + img.build;
                    
                    if ( connector.nfsroot ) {
                        img.nfsroot += '/' + connector.nfsroot;
                    }
                    
                    img.nfsopts = connector.nfsopts;
                }
                
                return ( img.nfsroot && img.nfsopts && ! img.hidden );
            });
            
            return self._renderScript('scripts/exports', {
                imgs: imgs
            }, function scriptRendered(err, result) {
                if ( err ) {
                    return next(err);
                }
                
                return next(null, result);
            });
        });
    },
    
    /**
     * Binary build
     */
    build: function (connector, params, next) {
        next = next || function () {};
        
        var conn = Connector.get(connector);
        
        if ( ! conn ) {
            // sails.log.error('Unkown connector: `', connector, '`.');
            return next('Unkown connector: `' + connector + '`.');
        }
        
        // Merge defaults with overwriten params
        _.merge(conn.env, params || {});
        
        sails.log.debug('Building with...', conn);
        
        var build = dateFormat(new Date(), conn.env.__build);
        
        return self._renderScript('scripts/connectors/' + conn.env.__template, {
            build: build,
            conn: conn
        }, function scriptRendered(err, result) {
            if ( err ) {
                return next(err);
            }
            
            return next(null, build, result);
        });
    }
});