/**
 * Bootstrap
 * (sails.config.bootstrap)
 *
 * An asynchronous bootstrap function that runs before your Sails app gets lifted.
 * This gives you an opportunity to set up your data model, run jobs, or perform some special logic.
 *
 * For more information on bootstrapping your app, check out:
 * http://links.sailsjs.org/docs/config/bootstrap
 */

module.exports.bootstrap = function(cb) {
    
    // Niobium config parameters
    var configParameters = sails.config.niobium.defaults;
    
    // All web distinct permissions
    var permissions = [
        {name: 'webAccess'},
        {name: 'viewUsers'},
        {name: 'manageUsers'},
        {name: 'viewRoles'},
        {name: 'manageRoles'},
        {name: 'viewPermissions'},
        {name: 'managePermissions'},
        {name: 'viewClients'},
        {name: 'manageClients'},
        {name: 'viewImages'},
        {name: 'manageImages'},
        {name: 'viewGroups'},
        {name: 'manageGroups'},
        {name: 'viewConnectors'},
        {name: 'manageConnectors'},
        {name: 'manageConfig'},
        
    ];
    
    var initializeBootstrap = function () {
        sails.log.info('~~~ Initializing Bootstrap ~~~');
        return createAndInjectConfig();
    };
    
    var abortBootstrap = function (err) {
        sails.log.error('An error has occurred:', err);
        return finalizeBootstrap();
    };
    
    var finalizeBootstrap = function () {
        sails.log.info('~~~ Finalizing bootstrap ~~~');
        return cb();
    };
    
    var createAndInjectConfig = function () {
        sails.log.info('Writing Niobium config...');
        
        Config.loadConfig(function (err) {
            if ( err ) {
                return abortBootstrap(err);
            }
            
            Config.setParams(configParameters, false, function (err) {
                if ( err ) {
                    return abortBootstrap(err);
                }
                
                return createOrUpdateRoleAdmin();
            });
        });
    };
    
    var createOrUpdateRoleAdmin = function () {
        sails.log.info('Creating `admin` role...');
        Role.findOrCreate({name: 'admin'}, {name: 'admin'})
        .populate('permissions')
        .exec(function roleFound(err, role) {
            if ( err ) {
                return abortBootstrap(err);
            }
            
            sails.log.info('Role created!');
            
            Permission.findOrCreateEach(['name'], permissions).exec(function permissionsFound(err, perms) {
                if ( err ) {
                    return abortBootstrap(err);
                }
                
                sails.log.info('Permissions created!');
                sails.log.info('Total permissions: ' + perms.length);
                
                if ( perms ) {
                    _.each(perms, function setPerm(perm) {
                        sails.log.info('Assigning `' + perm.name + '` permission to `' + role.name + '` role...');
                        role.permissions.add(perm.id);
                    });
                    role.save();
                    sails.log.info('Permissions assigned!');
                }
                
                sails.log.info('Creating `admin` user...');
                User.findOrCreate({name: 'admin'}, {
                    name: 'admin',
                    password: 'admin'
                }).exec(function userFound(err, user) {
                    if ( err ) {
                        return abortBootstrap(err);
                    }
                    
                    sails.log.info('User created!');
                    sails.log.info('Assigning `' + role.name + '` role to `' + user.name + '` user...');
                    user.roles.add(role.id);
                    user.save();
                    sails.log.info('Role assigned!');
                    
                    sails.log.info('');
                    return createOrUpdateRoleUser();
                });
            });
        });
    };
    
    var createOrUpdateRoleUser = function () {
        sails.log.info('Creating `user` role...');
        Role.findOrCreate({name: 'user'}, {name: 'user'})
        .populate('permissions')
        .exec(function roleFound(err, role) {
            if ( err ) {
                return abortBootstrap(err);
            }
            
            sails.log.info('Role created!');
            
            Permission.findOneByName('webAccess').exec(function permissionFound(err, perm) {
                if ( err ) {
                    return abortBootstrap(err);
                }
                
                if ( perm ) {
                    sails.log.info('Assigning `' + perm.name + '` permission to `' + role.name + '` role...');
                    role.permissions.add(perm.id);
                    role.save();
                    sails.log.info('Permission assigned!');
                }
                
                sails.log.info('Creating `user` user...');
                User.findOrCreate({name: 'user'}, {
                    name: 'user',
                    password: 'user'
                }).exec(function userFound(err, user) {
                    if ( err ) {
                        return abortBootstrap(err);
                    }
                    
                    sails.log.info('User created!');
                    sails.log.info('Assigning `' + role.name + '` role to `' + user.name + '` user...');
                    user.roles.add(role.id);
                    user.save();
                    sails.log.info('Role assigned!');
                    
                    sails.log.info('');
                    return loadConnectors();
                });
            });
        });
    };
    
    var loadConnectors = function () {
        sails.log.info('Loading connectors...');
        Connector.load(function connectorsLoaded (err) {
            if ( err ) {
                return abortBootstrap(err);
            }
            
            sails.log.info('');
            return finalizeBootstrap();
        });
    };
    
    return initializeBootstrap();
};
